@@ -11,18 +11,36 @@
 
 #include <KColorUtils>
 
+#ifdef Q_OS_WIN
+#include "windows.h"
+#endif
+
 const char *colorProperty = "KDE_COLOR_SCHEME_PATH";
 
+#ifdef Q_OS_WIN
+static bool isHighContrastModeActive()
+{
+    HIGHCONTRAST result;
+    result.cbSize = sizeof(HIGHCONTRAST);
+    if (SystemParametersInfo(SPI_GETHIGHCONTRAST, result.cbSize, &result, 0)) {
+        return (result.dwFlags & HCF_HIGHCONTRASTON);
+    }
+    return false;
+}
+#endif
+
 namespace Breeze
 {
 ToolsAreaManager::ToolsAreaManager()
     : QObject()
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QString path;
     if (qApp && qApp->property(colorProperty).isValid()) {
         path = qApp->property(colorProperty).toString();
     }
     recreateConfigWatcher(path);
+#endif
     configUpdated();
 }
 
@@ -30,9 +48,16 @@ ToolsAreaManager::~ToolsAreaManager()
 {
 }
 
+void ToolsAreaManager::loadSchemeConfig(const QString &path)
+{
+    const auto openFlags = path.isEmpty() ? KConfig::OpenFlag::FullConfig : KConfig::OpenFlag::NoGlobals;
+    _config = KSharedConfig::openConfig(path, openFlags);
+}
+
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 void ToolsAreaManager::recreateConfigWatcher(const QString &path)
 {
-    _config = KSharedConfig::openConfig(path);
+    loadSchemeConfig(path);
     if (!path.startsWith(QLatin1Char('/'))) {
         _watcher = KConfigWatcher::create(_config);
         connect(_watcher.data(), &KConfigWatcher::configChanged, this, &ToolsAreaManager::configUpdated);
@@ -40,48 +65,77 @@ void ToolsAreaManager::recreateConfigWatcher(const QString &path)
         _watcher.reset();
     }
 }
+#endif
 
-template<class T1, class T2>
-void appendIfNotAlreadyExists(T1 *list, T2 item)
+void ToolsAreaManager::appendIfNotAlreadyExists(const QMainWindow *window, const QPointer<QToolBar> &toolBar)
 {
-    for (auto listItem : *list) {
-        if (listItem == item) {
-            return;
+    const auto windowIt = std::find_if(_windows.begin(), _windows.end(), [window](const WindowToolBars &windowToolBars) {
+        return window == windowToolBars.window;
+    });
+    if (windowIt != _windows.end()) {
+        if (!windowIt->toolBars.contains(toolBar)) {
+            windowIt->toolBars.append(toolBar);
         }
+    } else {
+        _windows.emplace_back(WindowToolBars{window, {toolBar}});
+        connect(window, &QObject::destroyed, this, [this, window] {
+            removeWindow(window);
+        });
     }
-    list->append(item);
+}
+
+void ToolsAreaManager::removeWindowToolBar(const QMainWindow *window, const QPointer<QToolBar> &toolBar)
+{
+    const auto windowIt = std::find_if(_windows.begin(), _windows.end(), [window](const WindowToolBars &windowToolBars) {
+        return window == windowToolBars.window;
+    });
+    if (windowIt != _windows.end()) {
+        windowIt->toolBars.removeAll(toolBar);
+    }
+}
+
+void ToolsAreaManager::removeWindow(const QMainWindow *window)
+{
+    std::erase_if(_windows, [window](const WindowToolBars &windowToolBars) {
+        return window == windowToolBars.window;
+    });
 }
 
 void ToolsAreaManager::registerApplication(QApplication *application)
 {
     _listener = new AppListener(this);
     _listener->manager = this;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     if (application->property(colorProperty).isValid()) {
         auto path = application->property(colorProperty).toString();
         recreateConfigWatcher(path);
     }
+#endif
     application->installEventFilter(_listener);
     configUpdated();
 }
 
-QRect ToolsAreaManager::toolsAreaRect(const QMainWindow *window)
+QRect ToolsAreaManager::toolsAreaRect(const QMainWindow &window) const
 {
-    Q_ASSERT(window);
-
-    int itemHeight = window->menuWidget() ? window->menuWidget()->height() : 0;
-    for (auto item : _windows[window]) {
-        if (!item.isNull() && item->isVisible() && window->toolBarArea(item) == Qt::TopToolBarArea) {
-            itemHeight = qMax(item->mapTo(window, item->rect().bottomLeft()).y(), itemHeight);
+    int itemHeight = window.menuWidget() ? window.menuWidget()->height() : 0;
+    const auto windowIt = std::find_if(_windows.begin(), _windows.end(), [&window](const WindowToolBars &windowToolBars) {
+        return &window == windowToolBars.window;
+    });
+    if (windowIt != _windows.end()) {
+        for (auto item : windowIt->toolBars) {
+            if (!item.isNull() && item->isVisible() && window.toolBarArea(item) == Qt::TopToolBarArea) {
+                itemHeight = qMax(item->mapTo(&window, item->rect().bottomLeft()).y(), itemHeight);
+            }
         }
     }
     if (itemHeight > 0) {
         itemHeight += 1;
     }
 
-    return QRect(0, 0, window->width(), itemHeight);
+    return QRect(0, 0, window.width(), itemHeight);
 }
 
-bool ToolsAreaManager::tryRegisterToolBar(QPointer<QMainWindow> window, QPointer<QWidget> widget)
+bool ToolsAreaManager::tryRegisterToolBar(QPointer<const QMainWindow> window, QPointer<QWidget> widget)
 {
     Q_ASSERT(!widget.isNull());
 
@@ -92,14 +146,14 @@ bool ToolsAreaManager::tryRegisterToolBar(QPointer<QMainWindow> window, QPointer
 
     if (window->toolBarArea(toolbar) == Qt::TopToolBarArea) {
         widget->setPalette(palette());
-        appendIfNotAlreadyExists(&_windows[window], toolbar);
+        appendIfNotAlreadyExists(window, toolbar);
         return true;
     }
 
     return false;
 }
 
-void ToolsAreaManager::tryUnregisterToolBar(QPointer<QMainWindow> window, QPointer<QWidget> widget)
+void ToolsAreaManager::tryUnregisterToolBar(QPointer<const QMainWindow> window, QPointer<QWidget> widget)
 {
     Q_ASSERT(!widget.isNull());
 
@@ -110,17 +164,47 @@ void ToolsAreaManager::tryUnregisterToolBar(QPointer<QMainWindow> window, QPoint
 
     if (window->toolBarArea(toolbar) != Qt::TopToolBarArea) {
         widget->setPalette(window->palette());
-        _windows[window].removeAll(toolbar);
+        removeWindowToolBar(window, toolbar);
     }
 }
 
 void ToolsAreaManager::configUpdated()
 {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    if (qApp->property(colorProperty).isValid()) {
+        const auto colorSchemePath = qApp->property(colorProperty).toString();
+#ifdef Q_OS_WIN
+        // If no color scheme is set and high-contrast is active then use the system colors
+        if (colorSchemePath.isEmpty() && isHighContrastModeActive()) {
+            _config.reset();
+        } else
+#endif
+        {
+            if (!_config || _config->name() != colorSchemePath) {
+                loadSchemeConfig(colorSchemePath);
+            }
+        }
+    } else {
+#ifdef Q_OS_WIN
+        // If high-contrast is active then use the system colors
+        if (isHighContrastModeActive()) {
+            _config.reset();
+        } else
+#endif
+        {
+            loadSchemeConfig(QString{});
+        }
+    }
+#endif
     auto active = KColorScheme(QPalette::Active, KColorScheme::Header, _config);
     auto inactive = KColorScheme(QPalette::Inactive, KColorScheme::Header, _config);
     auto disabled = KColorScheme(QPalette::Disabled, KColorScheme::Header, _config);
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    _palette = qApp->palette();
+#else
     _palette = KColorScheme::createApplicationPalette(_config);
+#endif
 
     _palette.setBrush(QPalette::Active, QPalette::Window, active.background());
     _palette.setBrush(QPalette::Active, QPalette::WindowText, active.foreground());
@@ -129,15 +213,19 @@ void ToolsAreaManager::configUpdated()
     _palette.setBrush(QPalette::Inactive, QPalette::Window, inactive.background());
     _palette.setBrush(QPalette::Inactive, QPalette::WindowText, inactive.foreground());
 
-    for (auto window : _windows) {
-        for (auto toolbar : window) {
+    for (const WindowToolBars &windowToolBars : _windows) {
+        for (const auto &toolbar : windowToolBars.toolBars) {
             if (!toolbar.isNull()) {
                 toolbar->setPalette(_palette);
             }
         }
+
+        if (QMenuBar *menuBar = windowToolBars.window->menuBar()) {
+            menuBar->setPalette(_palette);
+        }
     }
 
-    _colorSchemeHasHeaderColor = KColorScheme::isColorSetSupported(_config, KColorScheme::Header);
+    _colorSchemeHasHeaderColor = _config ? KColorScheme::isColorSetSupported(_config, KColorScheme::Header) : false;
 }
 
 bool AppListener::eventFilter(QObject *watched, QEvent *event)
@@ -149,10 +237,12 @@ bool AppListener::eventFilter(QObject *watched, QEvent *event)
         return false;
     }
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    if (event->type() == QEvent::ApplicationPaletteChange) {
+        manager->configUpdated();
+    }
+#else
     if (event->type() == QEvent::DynamicPropertyChange) {
-        if (watched != qApp) {
-            return false;
-        }
         auto ev = static_cast<QDynamicPropertyChangeEvent *>(event);
         if (ev->propertyName() == colorProperty) {
             QString path;
@@ -163,6 +253,7 @@ bool AppListener::eventFilter(QObject *watched, QEvent *event)
             manager->configUpdated();
         }
     }
+#endif
 
     return false;
 }
@@ -173,7 +264,7 @@ bool ToolsAreaManager::eventFilter(QObject *watched, QEvent *event)
     Q_ASSERT(event);
 
     QPointer<QObject> parent = watched;
-    QPointer<QMainWindow> mainWindow = nullptr;
+    QPointer<const QMainWindow> mainWindow = nullptr;
     while (parent != nullptr) {
         if (qobject_cast<QMainWindow *>(parent)) {
             mainWindow = qobject_cast<QMainWindow *>(parent);
@@ -182,10 +273,17 @@ bool ToolsAreaManager::eventFilter(QObject *watched, QEvent *event)
         parent = parent->parent();
     }
 
-    if (QPointer<QMainWindow> mw = qobject_cast<QMainWindow *>(watched)) {
+    if (const QMainWindow *mw = qobject_cast<QMainWindow *>(watched)) {
         QChildEvent *ev = nullptr;
         if (event->type() == QEvent::ChildAdded || event->type() == QEvent::ChildRemoved) {
             ev = static_cast<QChildEvent *>(event);
+
+            if (event->type() == QEvent::ChildAdded) {
+                QChildEvent *childEvent = static_cast<QChildEvent *>(event);
+                if (QMenuBar *menuBar = qobject_cast<QMenuBar *>(childEvent->child())) {
+                    menuBar->setPalette(_palette);
+                }
+            }
         } else {
             return false;
         }
@@ -197,10 +295,10 @@ bool ToolsAreaManager::eventFilter(QObject *watched, QEvent *event)
 
         if (ev->added()) {
             if (mw->toolBarArea(tb) == Qt::TopToolBarArea) {
-                appendIfNotAlreadyExists(&_windows[mw], tb);
+                appendIfNotAlreadyExists(mw, tb);
             }
         } else if (ev->removed()) {
-            _windows[mw].removeAll(tb);
+            removeWindowToolBar(mw, tb);
         }
     } else if (qobject_cast<QToolBar *>(watched)) {
         if (!mainWindow.isNull()) {
@@ -216,13 +314,18 @@ void ToolsAreaManager::registerWidget(QWidget *widget)
     Q_ASSERT(widget);
     auto ptr = QPointer<QWidget>(widget);
 
-    QPointer<QMainWindow> mainWindow = qobject_cast<QMainWindow *>(ptr);
+    QPointer<const QMainWindow> mainWindow = qobject_cast<QMainWindow *>(ptr);
 
-    if (mainWindow && mainWindow == mainWindow->window()) {
+    if (mainWindow && mainWindow.data() == mainWindow->window()) {
         const auto toolBars = mainWindow->findChildren<QToolBar *>(QString(), Qt::FindDirectChildrenOnly);
         for (auto *toolBar : toolBars) {
             tryRegisterToolBar(mainWindow, toolBar);
         }
+
+        if (QMenuBar *menuBar = mainWindow->menuBar()) {
+            menuBar->setPalette(_palette);
+        }
+
         return;
     }
 
@@ -240,7 +343,7 @@ void ToolsAreaManager::registerWidget(QWidget *widget)
     if (mainWindow == nullptr) {
         return;
     }
-    if (mainWindow != mainWindow->window()) {
+    if (mainWindow.data() != mainWindow->window()) {
         return;
     }
     tryRegisterToolBar(mainWindow, widget);
@@ -251,12 +354,12 @@ void ToolsAreaManager::unregisterWidget(QWidget *widget)
     Q_ASSERT(widget);
     auto ptr = QPointer<QWidget>(widget);
 
-    if (QPointer<QMainWindow> window = qobject_cast<QMainWindow *>(ptr)) {
-        _windows.remove(window);
+    if (QPointer<const QMainWindow> window = qobject_cast<QMainWindow *>(ptr)) {
+        removeWindow(window);
         return;
     } else if (QPointer<QToolBar> toolbar = qobject_cast<QToolBar *>(ptr)) {
         auto parent = ptr;
-        QPointer<QMainWindow> mainWindow = nullptr;
+        QPointer<const QMainWindow> mainWindow = nullptr;
         while (parent != nullptr) {
             if (qobject_cast<QMainWindow *>(parent)) {
                 mainWindow = qobject_cast<QMainWindow *>(parent);
@@ -267,7 +370,7 @@ void ToolsAreaManager::unregisterWidget(QWidget *widget)
         if (mainWindow == nullptr) {
             return;
         }
-        _windows[mainWindow].removeAll(toolbar);
+        removeWindowToolBar(mainWindow, toolbar);
     }
 }
 
